unit DS042u1;

(*This program is written with a bias towards clarity (really) rather
  than robustness. If things are not right (e.g. Adapter type incorrectly
  specified in ini file) program tends just to report that and shut
  down, not to try to be "clever".

*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, IniFiles, PSD016eu, iBTMEXPW, ExtCtrls;

const ver='24 Aug 06';
      //Started 23 Aug 06, Essex, for monitor-by-broadband work.
      kDMLRomIndexMax=1;

type
  TDS042f1 = class(TForm)
    buReadTture0: TButton;
    buQuit: TButton;
    buReadTture1: TButton;
    laTture0: TLabel;
    laTture1: TLabel;
    paUnits: TPanel;
    rbCelsius: TRadioButton;
    rbFahrenheit: TRadioButton;
    procedure FormCreate(Sender: TObject);
    procedure buQuitClick(Sender: TObject);
    procedure buReadTture1Click(Sender: TObject);
    procedure buReadTture0Click(Sender: TObject);
    procedure FetchNDisplayTture(bWhich:byte);

  private
    { Private declarations }
    saDMLRomID:array [0..kDMLRomIndexMax] of string[16];
    (*While this example accesses only two chips, it is
      written so that extending it to more is easy.
      saDMLRomID[0] would hold 'A20000000547FB10' if
      the first chip's id were $A20000000547FB10. The
      address is given in the form returned by the
      iButtonViewer (software included in SDK), i.e.
      the right hand byte is the device ID byte.*)
    baDMLRomIDTable:array [0..kDMLRomIndexMax,0..7] of byte;
    (*baDMLRomIDTable[2,0] holds chip 2's first id byte*)
    baDMLRomIDofOne:array [0..7] of byte;
    (*A "temporary" variable, for passing ID of a ROM to
      external routine.
      baDMLRomIDofOne[0] holds a chip's first id byte*)
    sIniFileName:string;
    dfIniFile:TIniFile;
    iDMLPortNum,iDMLAdapterType,iDMLTmpResult:integer;
    (*iDMLTmpResult is a 'scratch' variable, never used for
         long-term storage of any value.*)
    liDMLSHandle:longint;

  sTmp:string;
  iErr:integer;
  //boTmp:boolean;
  bTmp:byte;
  rTmp:real;

  public
    { Public declarations }
   end;

var
  DS042f1: TDS042f1;

implementation

{$R *.DFM}
{$R+}//turn on range checking

procedure TDS042f1.FormCreate(Sender: TObject);
var c1:byte;

   procedure CreateIniFile;
   begin
   dfIniFile:=TIniFile.create(sIniFileName);
      dfIniFile.WriteString('adapter','type','you must put number here');
      dfIniFile.WriteString('adapter','port','you must put number here');
      dfIniFile.WriteString('chips','0','you must put 16 hex characters here');
      dfIniFile.WriteString('chips','1','you must put 16 hex characters here');
   dfIniFile.free;
   end;

   function boIDValid(sTmp:string):boolean;
   begin //should be expanded to see sTmp holds only hex characters
   //Be careful with changes to ini file part of program, or rogues
   //   xx or 'you' may no longer arise when they should.
   result:=true;
   if sTmp='xx' then result:=false;
   if copy(sTmp,1,3)='you' then result:=false;
   if (copy(sTmp,15,2)<>'10')and(copy(sTmp,15,2)<>'28') then
      begin
      showmessage('Unsuitable tture sensor chip. Family code must end 10 or 28');
      result:=false;
      end;
   end;

begin
DS042f1.caption:='DS042'+ver;

sIniFileName:=ExtractFileDir(application.exename);
sIniFileName:=sIniFileName+'\ds042ini.txt';

//Begin pick up info from ini file. (End is similarly marked.)
if not fileexists(sIniFileName) then begin
  CreateIniFile;
  end//no ; here
  else begin
   dfIniFile:=TIniFile.create(sIniFileName);
      sTmp:=dfIniFile.ReadString('adapter','type','xx');
      val(sTmp,iDMLAdapterType,iErr);
      if iErr<>0 then begin
        showmessage('The adapter type must be a number');
        application.terminate;
        end;

      sTmp:=dfIniFile.ReadString('adapter','port','yy');
      val(sTmp,iDMLPortNum,iErr);
      if iErr<>0 then begin
        showmessage('The adapter port must be a number');
        application.terminate;
        end;

      sTmp:=dfIniFile.ReadString('chips','0','xx');
      if not boIDValid(sTmp) then begin
        showmessage('The chip id of the first chip is not valid. '+
          'Must be 16 hex characters, and last must be tture sensor '+
          'family code.');
        application.terminate;
        end;
      saDMLRomID[0]:=sTmp;

      sTmp:=dfIniFile.ReadString('chips','1','xx');
      if not boIDValid(sTmp) then begin
        showmessage('The chip id of the second chip is not valid. '+
          'Must be 16 hex characters, and last must be tture sensor '+
          'family code.');
        application.terminate;
        end;
      saDMLRomID[1]:=sTmp;
    dfIniFile.free;

      for c1:=0 to kDMLRomIndexMax do begin
       sTmp:=saDMLRomID[c1];
         baDMLRomIDTable[c1,7]:=strtoint('$'+copy(sTmp,1,2));
         baDMLRomIDTable[c1,6]:=strtoint('$'+copy(sTmp,3,2));
         baDMLRomIDTable[c1,5]:=strtoint('$'+copy(sTmp,5,2));
         baDMLRomIDTable[c1,4]:=strtoint('$'+copy(sTmp,7,2));
         baDMLRomIDTable[c1,3]:=strtoint('$'+copy(sTmp,9,2));
         baDMLRomIDTable[c1,2]:=strtoint('$'+copy(sTmp,11,2));
         baDMLRomIDTable[c1,1]:=strtoint('$'+copy(sTmp,13,2));
         baDMLRomIDTable[c1,0]:=strtoint('$'+copy(sTmp,15,2));
         end;(*For c1:=...*)
   end;//else
//"Pick up info from ini file" section ends here.

end; //FormCreate

procedure TDS042f1.buQuitClick(Sender: TObject);
begin
application.terminate;
end;

procedure TDS042f1.buReadTture0Click(Sender: TObject);
begin
FetchNDisplayTture(0);
end;//buReadTture0Click

procedure TDS042f1.buReadTture1Click(Sender: TObject);
begin
FetchNDisplayTture(1);
end;//buReadTture1Click

procedure TDS042f1.FetchNDisplayTture(bWhich:byte);
var c1,bTmp1,bTmp2,bTmp3:byte;
    iTmp:integer;
begin
if bWhich=0 then laTture0.caption:='One moment...'//no ; here
 else  laTture1.caption:='One moment...';
//Execution of DMLGet1820... does take enough time that you
//  Really Should add this "frill".
application.processmessages;
//without this, you don't see the caption change,
//   (And even with it, you don't always see it!)
//   because
//   (at 24aug06) DMLGet18200... has a bad, Windows hogging,
//   loop as part of WaitABit. Hey! This works.... even though
//   it shouldn't. I'm not proud. If you can show me a better
//   way within Delphi 2, I'll be delighted!
// ---------------------

//Needed over and over...
   DMLEstablishSession(iDMLPortNum,iDMLAdapterType,bTmp,liDMLSHandle);

if bTmp<>0 then begin //1
    showmessage('DMLEstablishSession reported an error. Try again. '+
        'It may simply be that MicroLan was busy.');
    end// no ; here
  else begin //1

//First, put the ID of one chip (from table) into array for passing to Get1820...
//The first index of baDMLRomIDTable determines which chip will be read.
//For less flexibility, more speed, variables
//   baDMLRomIDofChip0, baDMLRomIDofChip1, type array[0..7] of byte
//   could be set up and used. This would save having to fill and
//   refill the "scratch" variable baDMLRomIDofOne.
    for c1:=0 to 7 do
    baDMLRomIDofOne[c1]:=baDMLRomIDTable[bWhich,c1];

    DMLGet1820Tture(bTmp1,bTmp2,bTmp3,iTmp,liDMLSHandle,baDMLRomIDofOne);

    if bTmp3<>0 then begin //2
      showmessage('Problem encountered reading DS1820. '+
          'TKB Error: '+inttostr(bTmp3)+' , Dallas error: '+
          inttostr(iTmp));
      rTmp:=-999;
    end//no ; here
    else begin //2
      if baDMLRomIDOfOne[0]=$10 then
         rTmp:=rRawToC(bTmp1,bTmp2);
      if baDMLRomIDOfOne[0]=$28 then
         rTmp:=rRawToC18B20(bTmp1,bTmp2);
     end;//else *2*

    if rbFahrenheit.checked then rTmp:=CtoF(rTmp);

    if bWhich=0 then laTture0.caption:=floattostrf(rTmp,ffFixed,5,2)//no ; here
      else  laTture1.caption:=floattostrf(rTmp,ffFixed,5,2)
end;//else *1*
end;//FetchNDisplayTture

end.
